<?php

require('metadata.php');

class Action {
    var $name;
    var $version;
    var $errors = array();
    var $log = array();
    var $subactions = array();
    function Action($name, $version) {
        $this->name = $name;
        $this->version = $version;
    }
    function perform() {
        foreach($this->subactions as $action) {
            if (count($this->errors) > 0) return;
            $action->perform();
            foreach($action->errors as $error) $this->errors[] = $error;
            foreach($action->log as $log) $this->log[] = " - ".$log;
        }
    }
    function start_log() {
        $this->log[] = $this->name." v".$this->version;
    }
    function write_file($filename, $content) {
        $this->log[] = " - Creating file: ".$filename;
        $file = fopen($filename, "w");
        foreach($content as $key => $value) {
            fputs($file, $value."\r\n");
        }
        fclose($file);
        $this->log[] = " - File created";
    }
    function get_file_content($filename, $mappings) {
        $this->log[] = " - Parsing file: ".$filename;
        $contents = file_get_contents($filename);
        $this->log[] = " - Scanning content...";
        $results = array();
        foreach($mappings as $key => $value) {
            preg_match_all($value, $contents, $matches);
            $results[$key] = $matches[1];
            $this->log[] = " - Values found for ".$key.": ".count($matches[1]);
        }
        $this->log[] = " - File parsed";
        return $results;
    }
    function check_param($param, $results, $number, $checktype) {
        $count = count($results[$param]);
        $error = false;
        switch ($checktype) {
            case "EQ": $error = $count <> $number; break;
        }
        if ($error)
            $this->errors[] = "Wrong number of ".$param." tags";
    }
}

class NewsUploadAction extends Action {
    var $meta;
    function NewsUploadAction($meta) {
        parent::Action("News upload action", "1.0.0");
        $this->meta = $meta;
    }
    function perform() {
        parent::start_log();
        $this->log[] = "";
        $this->log[] = "Starting to insert news...";
        $this->meta->upload_news();
        $this->log[] = "News added.";
        $this->log[] = "";
        $this->log[] = "Action finished.";
    }
}

class NFOParserAction extends Action {
    var $file;
    function NFOParserAction($file) {
        parent::Action("NFO parser action", "2.0.0");
        $this->file = $file;
    }
    function get_mappings($scheme) {
        switch ($scheme) {
            case "fwyh":
                $mappings = array(
                    "ARTIST:" => "/ARTiST\.\.\.\.\[ (.*)[ ]{3}/",
                    "RELEASE:" => "/TiTLE\.\.\.\.\.\[ (.*)[ ]{3}/",
                    "LABEL:" => "/LABEL\.\.\.\.\.\[ (.*)[ ]{3}/",
                    "YEAR:" => "/REL\.DATE\.\.\[ .*-.*-([0-9]{4})/",
                    "TRACKNR:" => "/[ ]([0-9]{2}).*[ ].*[ ]{2,}.*[0-9:]{5}/",
                    "TRACKNAME:" => "/[ ][0-9]{2}.*?[ ](.*)[ ].*[ ][0-9:]{5}/",
                    "TRACKTIME:" => "/[ ][0-9]{2}.*?[ ].*[ ].*[ ]([0-9:]{5})/"
                );
                break;
            case "amok":
                $mappings = array(
                    "ARTIST:" => "/[ ]*?(.*) - .* \(c\) [0-9]{4}/",
                    "RELEASE:" => "/[ ]*?.* - (.*) \(c\) [0-9]{4}/",
                    "LABEL:" => "/Label\.\.\.\.\.: (.*)/",
                    "YEAR:" => "/[ ]*?.* - .* \(c\) ([0-9]{4})/",
                    "TRACKNR:" => "/[ ]{2}([0-9]{2})[ ].*[0-9:]{5}/",
                    "TRACKNAME:" => "/[ ]{2}[0-9]{2}[ ](.*)[0-9:]{5}/",
                    "TRACKTIME:" => "/[ ]{2}[0-9]{2}[ ].*([0-9:]{5})/"
                );
                break;
            case "amok2":
                $mappings = array(
                    "ARTIST:" => "/Artist\.*: (.*)/",
                    "RELEASE:" => "/Album\.*: (.*)/",
                    "LABEL:" => "/Label\.*: (.*)/",
                    "YEAR:" => "/Rel\.Date\.\.: [0-9]{2}-[0-9]{2}-([0-9]{4})/",
                    "TRACKNR:" => "/([0-9]{2})[ ].*[ ]{3,}[0-9:]{5}/",
                    "TRACKNAME:" => "/[0-9]{2}[ ](.*)[ ]{3,}[0-9:]{5}/",
                    "TRACKTIME:" => "/[0-9]{2}[ ].*[ ]{3,}([0-9:]{5})/"
                );
                break;
            case "vce":
                $mappings = array(
                    "ARTIST:" => "/Artist[\.]*(.*)\r/",
                    "RELEASE:" => "/Album[\.]*(.*)\r/",
                    "LABEL:" => "/Label[\.]*(.*)\r/",
                    "YEAR:" => "/Street.*\/([0-9]{4})/",
                    "TRACKNR:" => "/([0-9]{2})\.[ ]{3}.*[ ]{2}[0-9:]{5}/",
                    "TRACKNAME:" => "/[0-9]{2}\.[ ]{3}(.*)[ ]{2}[0-9:]{5}/",
                    "TRACKTIME:" => "/[0-9]{2}\.[ ]{3}.*[ ]{2}([0-9:]{5})/"
                );
                break;
            case "radial":
                $mappings = array(
                    "ARTIST:" => "/:[ ]{4,}(.*) - .*[ ]{4,}:\r/",
                    "RELEASE:" => "/:[ ]{4,}.* - (.*)[ ]{4,}:\r/",
                    "LABEL:" => "/Label.*: (.{19})/",
                    "YEAR:" => "/Year.*: ([0-9]{4})/",
                    "TRACKNR:" => "/[ ]{3}([0-9]{2})[ ]{6}.*[ ]{2}[0-9:]{5}/",
                    "TRACKNAME:" => "/[ ]{3}[0-9]{2}[ ]{6}(.*)[ ]{2}[0-9:]{5}/",
                    "TRACKTIME:" => "/[ ]{3}[0-9]{2}[ ]{6}.*[ ]{2}([0-9:]{5})/"
                );
                break;
            case "pyt":
                $mappings = array(
                    "ARTIST:" => "/artist      : (.*)\r/",
                    "RELEASE:" => "/title       : (.*)\r/",
                    "LABEL:" => "/label       : (.*)/",
                    "YEAR:" => "/date        : (.*)/",
                    "TRACKNR:" => "/([0-9]{2})-.*[ ]{3,}[0-9:]{5}/",
                    "TRACKNAME:" => "/[0-9]{2}-(.*)[ ]{3,}[0-9:]{5}/",
                    "TRACKTIME:" => "/[0-9]{2}-.*[ ]{3,}([0-9:]{5})/"
                );
                break;
            case "cmg":
                $mappings = array(
                    "ARTIST:" => "/Artist \.\.\.\.\.\.\.\.: (.*)\r/",
                    "RELEASE:" => "/Title \.\.\.\.\.\.\.\.\.: (.*)\r/",
                    "LABEL:" => "/Label \.\.\.\.\.\.\.\.\.: (.*)\r/",
                    "YEAR:" => "/Release Date \.\.: .*-([0-9]{4})/",
                    "TRACKNR:" => "/([0-9]{2})  :::  .*  :::  [0-9:]{5}/",
                    "TRACKNAME:" => "/[0-9]{2}  :::  (.*)  :::  [0-9:]{5}/",
                    "TRACKTIME:" => "/[0-9]{2}  :::  .*  :::  ([0-9:]{5})/"
                );
                break;
            default:
                $mappings = array(
                    "ARTIST:" => "/Artist  : (.*)/",
                    "RELEASE:" => "/Title   : (.*)/",
                    "LABEL:" => "/Label   : (.*)/",
                    "YEAR:" => "/Year    : ([0-9]{4})/",
                    "TRACKNR:" => "/([0-9]{2})\..*[ ]{3,}[0-9:]{5}/",
                    "TRACKNAME:" => "/[0-9]{2}\.(.*)[ ]{3,}[0-9:]{5}/",
                    "TRACKTIME:" => "/[0-9]{2}\..*[ ]{3,}([0-9:]{5})/"
                );
                break;
        }
        return $mappings;
    }
    function get_content($file, $results) {
        $content = array(
            "COVER:" => "cover.jpg",
            "FORMAT:" => "CD",
            "RELEASETYPE:" => "Album",
            "ARTIST_LINK:" => ""
        );
        // special cases - we can get some info from filename
        $releaseformats = array(
            "CDM" => "CD5",
            "CDS" => "CD5",
            "2CD" => "2CD",
            "MC" => "CS"
        );
        $releasetypes = array(
            "CDM" => "Single",
            "CDS" => "Single",
            "2CD" => "Album",
            "MC" => "Album"
        );
        $releasetypepatterns = "/.*-(cdm|cds|2cd|mc)-.*/";
        if (preg_match($releasetypepatterns, $file, $matches)) {
            $content["RELEASETYPE:"] = $releasetypes[strtoupper($matches[1])];
            $content["FORMAT:"] = $releaseformats[strtoupper($matches[1])];
        }
        $normalitems = array("ARTIST:", "RELEASE:", "LABEL:", "YEAR:");
        foreach ($normalitems as $item) {
            $content[$item] = $this->process_param($item, $results[$item][0]);
        }
        $finalcontent = array();
        foreach ($content as $key => $value) {
            $finalcontent[] = $key.$value;
        }
        $i = 0;
        $finalcontent[] = "";
        foreach ($results['TRACKNR:'] as $value) {
            $finalcontent[] = "TRACKNR:".$results['TRACKNR:'][$i];
            $finalcontent[] = "TRACKNAME:".$this->process_param("TRACKNAME", $results['TRACKNAME:'][$i]);
            $finalcontent[] = "TRACKTIME:".$results['TRACKTIME:'][$i];
            $i++;
        }
        $finalcontent[] = "";
        $finalcontent[] = "REVIEW:";
        $finalcontent[] = "AUTHOR:Lic";
        return $finalcontent;
    }
    function process_param($param, $value) {
        switch ($param) {
            case "LABEL:":
                $value = str_replace("Records", "", $value);
                $value = str_replace("Recordings", "", $value);
                break;
        }
        return trim($value);
    }
    function perform() {
        parent::start_log();
        $this->log[] = "";
        $this->log[] = "Starting to parse nfo file...";
        $this->log[] = "File: ".$this->file;
        preg_match("/.*-(.*).nfo/i", $this->file, $matches);
        $scheme = strtolower($matches[1]);
        if ($scheme == "") {
            $this->errors[] = "No scheme specified";
            return;
        }
        if (!file_exists($this->file)) {
            $this->errors[] = "No nfo file";
            return;
        }
        $this->log[] = "Scheme: ".$scheme;
        $this->log[] = "";
        $results = $this->get_file_content($this->file, $this->get_mappings($scheme));
        $this->log[] = "";
        $contents = $this->get_content($this->file, $results);
        $this->write_file("new_review.txt", $contents);
        $this->log[] = "";
        $this->log[] = "Download prepared file: <a href=\"new_review.txt\">new_review.txt</a>";
        $this->log[] = "";
        unlink($this->file);
        $this->log[] = "Parsed nfo removed.";
        $this->log[] = "";
        $this->log[] = "Action finished.";
    }
}

class ReviewUploadAction extends Action {
    var $meta;
    var $file;
    function ReviewUploadAction($meta) {
        parent::Action("Upload review action", "2.0.0");
        $this->meta = $meta;
        $this->file = "new_review.txt";
    }
    function get_mappings() {
        return array(
            "ARTIST" => "/ARTIST:(.{1,})\r/",
            "ARTIST_BASE" => "/ARTIST_BASE:(.{1,})\r/",
            "RELEASE" => "/RELEASE:(.{1,})\r/",
            "YEAR" => "/YEAR:(.{1,})\r/",
            "FORMAT" => "/FORMAT:(.{1,})\r/",
            "COVER" => "/COVER:(.{1,})\r/",
            "RELEASETYPE" => "/RELEASETYPE:(.{1,})\r/",
            "DISK" => "/DISK:[ ]??(.{1,})\r/",
            "DISKFORMAT" => "/DISK:[ ]??(.{1,})\r/",
            "TRACKNR" => "/TRACKNR:[ ]??(.{1,})\r/",
            "TRACKNR" => "/TRACKNR:[ ]??(.{1,})\r/",
            "TRACKNAME" => "/TRACKNAME:[ ]??(.{1,})\r/",
            "TRACKTIME" => "/TRACKTIME:[ ]??([0-9:]{4,5})\r/",
            "REVIEW" => "/REVIEW:[ ]??(.{1,})\r/",
            "AUTHOR" => "/AUTHOR:[ ]??(.{1,})\r/",
            "LABEL" => "/LABEL:(.{1,})\r/",
            "PLANID" => "/PLANID:(.{1,})\r/",
            "PLANREV" => "/PLANREV:(.{1,})\r/",
            "REVIEWID" => "/REVIEWID:(.{1,})\r/",
            "NEWS" => "/NEWS:(.{1,})\r/",
            "ARTIST_LINK" => "/ARTIST_LINK:(.{1,})\r/"
        );
    }
    function get_rss_content() {
        $rss = array(
            "<?xml version=\"1.0\" ?>",
            "<rss version=\"2.0\">",
            "<channel>",
            "<title>industrial.onego.ru: Reviews Archive RSS Feed</title>",
            "<description>Industrial music reviews by the programmer.</description>",
            "<link>http://industrial.onego.ru</link>"
        );
        $this->meta->get_news();
        foreach ($this->meta->last as $release) {
            $rss[] = "<item>";
            $date = date("d.m.Y", strtotime($release->date));
            $artist = $release->show_title;
            $releasetitle = $release->title;
            $reviewouid = $release->ouid;
            $rss[] = "<title>".$date.": ".$artist." - ".$releasetitle."</title>";
            $rss[] = "<description>New review is posted on site</description>";
            $rss[] = "<link>http://industrial.onego.ru/reviews/main.php?mode=review&amp;id=".$reviewouid."</link>";
            $rss[] = "</item>";
            $rss[] = "</channel>";
            $rss[] = "</rss>";
        }
        return $rss;
    }
    function perform() {
        parent::start_log();
        $this->log[] = "";
        if (!file_exists("new_review.txt")) {
            $this->errors[] = "File 'new_review.txt' not found";
            return;
        }
        $results = $this->get_file_content($this->file, $this->get_mappings());
        $this->log[] = "";
        $this->check_param("ARTIST", $results, 1, "EQ");
        $this->check_param("RELEASE", $results, 1, "EQ");
        $this->check_param("YEAR", $results, 1, "EQ");
        $this->check_param("FORMAT", $results, 1, "EQ");
        $this->check_param("RELEASETYPE", $results, 1, "EQ");
        $this->check_param("LABEL", $results, 1, "EQ");
        $this->check_param("TRACKNAME", $results, count($results["TRACKNR"]), "EQ");
        $this->check_param("TRACKTIME", $results, count($results["TRACKNR"]), "EQ");
        $this->check_param("REVIEW", $results, 1, "EQ");
        $this->check_param("AUTHOR", $results, 1, "EQ");
        $this->check_param("PLANID", $results, count($results["PLANREV"]), "EQ");
        if (count($this->errors) > 0) return;
        $this->meta->start_transaction();
        $artist = trim($results["ARTIST"][0]);
        $base_artist = $artist;
        if (count($results["ARTIST_BASE"]) > 0)
            $base_artist = $results["ARTIST_BASE"][0];
        $letter = strtoupper(substr($base_artist, 0, 1));
        $artistouid = $this->meta->set_artist($artist, $base_artist, $letter);
        $this->log[] = "Artist parsed: ".$artist." (".$base_artist.")";
        $label = trim($results["LABEL"][0]);
        $labelouid = $this->meta->set_label($label);
        $this->log[] = "Label parsed: ".$label;
        $release = trim($results["RELEASE"][0]);
        $format = trim($results["FORMAT"][0]);
        $year = trim($results["YEAR"][0]);
        $releasetype = trim($results["RELEASETYPE"][0]);
        if ($release == "" || $format == "" || $year == "" || $releasetype == "") {
            $this->errors[] = "Release title, format, year or type are empty";
            $this->meta->rollback();
            return;
        }
        $releaseouid = $this->meta->set_release($release, $year, $format, $artistouid, $releasetype, $labelouid);
        $this->log[] = "Release parsed: ".$release.", ".$format.", ".$year;
        $i = 0;
        $diskid = 0;
        $tracknumbers = array();
        foreach ($results["TRACKNR"] as $tracknr) {
            if ($i == 0 || in_array($tracknr, $tracknumbers)) {
                $disktitle = "";
                if (count($results["DISK"]) > $diskid) $disktitle = trim($results["DISK"][$diskid]);
                $diskformat = "";
                if (count($results["DISKFORMAT"]) > $diskid) $diskformat = trim($results["DISKFORMAT"][$diskid]);
                $diskouid = $this->meta->set_disk($releaseouid, $disktitle, $diskid, $diskformat);
                $tracknumbers = array();
                $diskid++;
                $this->log[] = "Disk parsed: ".$diskid." ".$disktitle;
            }
            $tracknumbers[] = $tracknr;
            $trackname = trim($results["TRACKNAME"][$i]);
            $tracktimestr = trim($results["TRACKTIME"][$i]);
			$pos = strpos($tracktimestr, ":");
			$tracktime = substr($tracktimestr, 0, $pos) * 60 + substr($tracktimestr, $pos + 1);
			$this->meta->set_track($trackname, $tracktime, $diskouid, $tracknr);
			if (count($this->meta->dataconnection->errors) > 0) return;
			$this->log[] = "Track parsed: ".$tracknr.". ".$trackname." (".$tracktimestr.")";
            $i++;
        }
        $reviewtext = $results["REVIEW"][0];
        $reviewdate = date("Y.m.d H:i:s", time());
        $author = $results["AUTHOR"][0];
        $this->meta->set_review($releaseouid, $reviewtext, $reviewdate, $author);
        $this->log[] = "Review parsed: ".$reviewtext."; ".$reviewdate." by ".$author;
        $i = 0;
        foreach ($results["PLANID"] as $planid) {
            $planreview = $results["PLANREV"][$i];
            $this->meta->set_plan($planid, $planreview);
            $this->log[] = "Planned review parsed: ".$planreview;
            $i++;
        }
        foreach ($results["REVIEWID"] as $reviewid) {
            $this->meta->delete_plan_by_id($reviewid);
            $this->log[] = "Planned review deleted: ".$reviewid;
        }
        foreach ($results["NEWS"] as $news) {
            $newstext = $results["NEWS"];
            $newsdate = date("Y.m.d H:i:s", time());
            $this->meta->set_news($newstext, $newsdate);
            $this->log[] = "News parsed: ".$newstext."; ".$newsdate;
        }
        foreach ($results["ARTIST_LINK"] as $artistlink) {
            $this->meta->set_artist_link($artistouid, $artistlink);
            $this->log[] = "Artist link parsed: ".$artistlink;
        }
        $release = $artist." \"".$release."\"";
        $this->meta->delete_plan($release);
        $this->log[] = "Release removed from plan: ".$release;
        $this->meta->commit_transaction();
        $this->log[] = "Transaction commited";
        $this->write_file("feed.rss", $this->get_rss_content());
        $this->log[] = "RSS written";
        if (count($results["COVER"]) > 0) {
    		$coverfilename = $results["COVER"][0];
    		if(strpos(strtolower($coverfilename),".jpg")) $extension = ".jpg";
    		if(strpos(strtolower($coverfilename),".gif")) $extension = ".gif";
    		if(strpos(strtolower($coverfilename),".bmp")) $extension = ".bmp";
    		if(file_exists($coverfilename)) {
    			$coverfilename = str_replace(".jpg", "", $coverfilename);
    			$coverfilename = str_replace(".gif", "", $coverfilename);
    			$coverfilename = str_replace(".bmp", "", $coverfilename);
    			$newfilename = "".$releaseouid;
    			while(strlen($newfilename)<8) $newfilename = "0".$newfilename;
    			copy($coverfilename.$extension, "./covers/".$newfilename.$extension);
    			$this->log[] = "Cover ".$newfilename.$extension." copied";
                unlink($coverfilename.$extension);
                $this->log[] = "File ".$coverfilename.$extension." removed";
    		}
		}
        unlink("new_review.txt");
        $this->log[] = "File new_review.txt removed";
        $this->log[] = "Action finished";
    }
}

class ChangesAction extends Action {
    var $file;
    var $meta;
    function ChangesAction($meta) {
        parent::Action("Changes action", "3.0.0");
        $this->file = "changes.txt";
        $this->meta = $meta;
    }
    function get_mappings() {
        return array(
            "QUERY" => "/QUERY:(.{1,})/",
            "DESC" => "/DESC:(.{1,})/"
        );
    }
    function perform() {
        parent::start_log();
        $this->log[] = "";
        $this->log[] = "Performing subactions:";
        $results = $this->get_file_content($this->file, $this->get_mappings());
        $this->check_param("DESC", $results, count($results["QUERY"]), "EQ");
        if (count($this->errors) > 0) return;
        $this->meta->start_transaction();
        $i = 0;
        $this->log[] = "";
        $this->log[] = "Executing queries:";
        foreach($results["QUERY"] as $query) {
            if (count($this->meta->dataconnection->errors) == 0) {
                $this->log[] = " - Executing: ".$results["DESC"][$i];
                $this->meta->execute_query($query);
            }
            $i++;
        }
        $this->meta->commit_transaction();
        $this->log[] = "";
        unlink($this->file);
        $this->log[] = "Changes file removed";
        $this->log[] = "";
        $this->log[] = "Action finished.";
    }
}

class ActionsAction extends Action {
    var $meta;
    function ActionsAction($meta, $password) {
        parent::Action("Actions action", "1.0.0");
        $this->meta = $meta;
        if (md5($password) != get_upload_password())
            $this->errors[] = "Wrong password for the action";
    }
    function perform() {
        if (count($this->errors) > 0) return;
        parent::start_log();
        $this->meta->start_transaction();
        foreach ($this->meta->actions as $action) {
            $this->log[] = "Executing: ".$action->name;
            $this->meta->execute_query($action->query);
            if (count($this->meta->errors) > 0) return;
        }
        $this->meta->commit_transaction();
        $this->log[] = "";
        $this->log[] = "Action finished";
    }
}

class ActionFactory {
    var $dbconnection;
    function ActionFactory($dbconnection) {
        $this->dbconnection = $dbconnection;
    }
    function get_action($action_name) {
        switch ($action_name) {
            case "news":
                $meta = new UploadNewsMeta($this->dbconnection, $_REQUEST['newstext'], $_REQUEST['photofile'], $_REQUEST['password']);
                return new NewsUploadAction($meta);
                break;
            case "nfo":
                return new NFOParserAction($_REQUEST["file"]);
                break;
            case "changes":
                $meta = new ChangesMeta($this->dbconnection);
                return new ChangesAction($meta);
                break;
            case "review":
                $meta = new NewReviewMeta($this->dbconnection);
                return new ReviewUploadAction($meta);
                break;
            case "actions":
                $meta = new ActionsMeta($this->dbconnection);
                return new ActionsAction($meta, $_REQUEST["password"]);
                break;
        }
    }
}

?>
