<?php

class Element {
	var $zone = "";
	var $errors = array();
	function Element($zone = "") {
		$this->zone = $zone;
	}
	function prepare() {}
}

class MetadataElement extends Element {
    var $metadata;
    function MetadataElement($metadata, $zone = "") {
        parent::Element($zone);
        $this->metadata = $metadata;
    }
}

class ImageElement extends Element {
    var $image;
    var $description;
	function ImageElement($zone, $imagepath, $description = "") {
		parent::Element($zone);
		$this->image = new Image($imagepath);
		$this->description = $description;
	}
	function prepare(&$template) {
		$template->assign_vars(array(
			"SRC" => $this->image->path,
			"WIDTH" => $this->image->width,
			"HEIGHT" => $this->image->height,
			"ALT" => $this->description
		));
		$template->assign_var_from_handle($this->zone."IMAGE", "image");
		$template->assign_var($this->zone."IMAGE", 
			preg_replace("/\n/", "", $template->_tpldata['.'][0][$this->zone."IMAGE"]));
	}
}

class CoverElement extends ImageElement {
	function CoverElement($zone, $imagepath, $description = "") {
		Element::Element($zone);
		$this->image = new CoverImage($imagepath);
		$this->description = $description;
	}
}

class MenuElement extends Element {
	var $menu;
	function MenuElement($zone, $menu) {
		parent::Element($zone);
		$this->menu = $menu;
	}
	function prepare(&$template) {
        foreach ($this->menu->items as $data) {
            if ($data->address != "") {
                $template->assign_block_vars($this->zone."MENU", array(
                    "LINK" => "<a href=\"".$data->address."\">".$data->name."</a>",
                    "DESCRIPTION" => $data->description,
                    "DELIMITER" => $data->delimiter
                ));
            } else {
                $template->assign_block_vars($this->zone."MENU", array(
                    "LINK" => $data->name,
                    "DESCRIPTION" => $data->description,
                    "DELIMITER" => $data->delimiter
                ));
            }
        }
	}
}

class StatisticsElement extends MetadataElement {
	function StatisticsElement($metadata) {
		parent::MetadataElement($metadata, "");
	}
	function prepare(&$template) {
		$template->assign_vars(array(
			"REVIEWS" => $this->metadata->reviews,
			"ARTISTS" => $this->metadata->artists,
			"LABELS" => $this->metadata->labels
		));
	}
}

class ListReviewsElement extends MetadataElement {
	function ListReviewsElement($metadata, $zone = "") {
		parent::MetadataElement($metadata, $zone);
	}
	function prepare(&$template) {
        $data = $this->metadata->data[0];
		$covername = $data->releaseouid;
		$release = $data->releasetitle;
        $template->assign_vars(array(
            $this->zone."LISTMODE" => $data->listmode,
		    $this->zone."LISTBY" => $data->listby
        ));
		$coverelement = new CoverElement("COVER", $covername, "&quot;".$release."&quot; cover");
		$coverelement->prepare($template);
		foreach ($this->metadata->data as $data) {
    		$template->assign_block_vars($this->zone."LIST", array(
                "DATE" => date("d.m.Y", strtotime($data->date)),
                "LINK_ARTIST" => "<a href=\"main.php?mode=reviews&id=".$data->artistsouid."\">".$data->artisttitle."</a>",
                "ARTIST" => $data->artisttitle,
                "LINK_RELEASE" => "<a href=\"main.php?mode=review&id=".$data->reviewouid."\">".$data->releasetitle."</a>",
                "RELEASE" => $data->releasetitle,
                "TYPE" => $data->type,
                "FORMAT" => $data->format,
                "YEAR" => $data->year
            ));
        }
	}
}

class ReviewsPlanElement extends MetadataElement {
	function ReviewsPlanElement($metadata, $zone = "") {
		parent::MetadataElement($metadata, $zone);
	}
	function prepare(&$template) {
        $i = 0;
        $review_plan = "";
		while ($i < count($this->metadata->reviews)) {
			$review_plan .= ($review_plan==""?"":"; ").$this->metadata->reviews[$i]->release;
			$i++;
		}
		if($review_plan=="") $review_plan = "Nothing is planned currently...";
		$template->assign_vars(array(
			"REVIEW_PLAN" => $review_plan)
		);
	}
}

class NewsElement extends MetadataElement {
    function NewsElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        if (count($this->metadata->news) == 0) return;
        $template->assign_block_vars($this->zone.'NEWS', array(
            "DUMMY" => "_"
        ));
        $imagename = $this->metadata->news[0]->image;
        if ($imagename != "") {
            $template->assign_block_vars($this->zone.'NEWS.NEWSIMAGESECTION', array(
                "DUMMY" => "_"
            ));
            $image = new ImageElement("NEWS", $imagename);
            $image->prepare($template);
        }
        foreach ($this->metadata->news as $data)
			$template->assign_block_vars($this->zone.'NEWS.list', array(
				"BODY" => $data->body,
				"DATE" => date("d.m.Y", strtotime($data->date))
			));
    }
}

class DiscographiesElement extends MetadataElement {
    function DiscographiesElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        $i = 1;
        foreach ($this->metadata->discogs as $data) {
			$fileinfo = new FileInfo("discographies/".str_replace(" ","_",strtolower($data->band)).".txt");
			$template->assign_block_vars('discos', array(
				"LINK" => $fileinfo->path,
				"ARTIST" => $data->band,
				"DATE" => $fileinfo->modificationdate,
				"DELIMITER" => $i == count($this->metadata->discogs) ? "" : "<br>")
			);
			$i++;
		}
    }
}

class NostalgiaElement extends ImageElement {
    function NostalgiaElement($metadata, $zone = "") {
		$photonumber = rand(1, count($metadata->photos))-1;
		$photo = $metadata->photos[$photonumber]->photo;
		$description = $metadata->photos[$photonumber]->description;
		$place = $metadata->photos[$photonumber]->place;
        parent::ImageElement($zone, "photos/".$photo, $place.". ".$description);
    }
}

class AuthorPhotoElement extends ImageElement {
    function AuthorPhotoElement($zone = "") {
		$dir = "iorimages";
		$dh  = opendir($dir);
		while (false !== ($filename = readdir($dh))) {
   			$files[] = $filename;
		}
		$countphotos = 0;
		for($i = 2; $i<count($files); $i++) {
			if(preg_match("/my_photo[0-9]{3}/", $files[$i])) $countphotos++;
		}
		$photonumber = rand(1, $countphotos);
		while(strlen($photonumber)<3) $photonumber = "0".$photonumber;
        parent::ImageElement($zone, "iorimages/my_photo".$photonumber.".jpg", "Author");
    }
}

class ErrorElement extends Element {
    var $errors;
    function ErrorElement($zone, $errors) {
        parent::Element($zone);
        $this->errors = $errors;
    }
    function prepare(&$template) {
        foreach ($this->errors as $text) {
            $template->assign_block_vars($this->zone."ERRORS", array(
                "TEXT" => $text
            ));
        }
    }
}

class FileInfoElement extends Element {
    var $fileinfo;
    function FileInfoElement($path, $zone = "") {
        parent::Element($zone);
        $this->fileinfo = new FileInfo($path);
    }
    function prepare(&$template) {
        $template->assign_vars(array(
            $this->zone."DATE" => $this->fileinfo->modificationdate
        ));
    }
}

class ThanksElement extends MetadataElement {
    function ThanksElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        foreach ($this->metadata->thanks as $data) {
            if($data->url != null) {
				$template->assign_block_vars($this->zone.'THANKSURL', array(
					"URL" => $data->url,
					"FULL_NAME" => $data->FullName,
					"WHAT_FOR" => $data->WhatFor)
				);
			} else {
				$template->assign_block_vars('THANKSNOURL', array(
					"FULL_NAME" => $data->FullName,
					"WHAT_FOR" => $data->WhatFor)
				);
			}
        }
    }
}

class FAQElement extends MetadataElement {
    function FAQElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        foreach ($this->metadata->faq as $data) {
            $template->assign_block_vars($this->zone."FAQ", array(
                "QUESTION" => $data->question,
                "ANSWER" => $data->answer
            ));
        }
    }
}

class ReleaseHeaderElement extends MetadataElement {
    function ReleaseHeaderElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        $data = $this->metadata->info[0];
		$urelease = strtoupper($data->releasetitle);
		$urelease = str_replace("&AUML;","&Auml;",$urelease);
		$urelease = str_replace("&UUML;","&Uuml;",$urelease);
		$urelease = str_replace("&OUML;","&Ouml;",$urelease);
        $template->assign_vars(array(
            "ARTIST" => "<a href=\"main.php?mode=reviews&id=".$data->artistouid."\">".$data->artisttitle."</a>",
            "RELEASE" => $urelease,
        ));
    }
}

class ReleaseInfoElement extends MetadataElement {
    function ReleaseInfoElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        $template->assign_block_vars($this->zone."RELINFO", array(
            "DUMMY" => "_"
        ));
        $data = $this->metadata->info[0];
        $coverelement = new CoverElement("COVER", $data->releaseouid, "&quot;".$data->releasetitle."&quot; cover");
        $coverelement->prepare($template);
        $template->assign_vars(array(
            "LABEL" => $data->labeltitle,
            "LABEL_LINK" => "main.php?mode=reviews&type=label&id=".$data->labelouid,
            "YEAR" => "<a href=\"main.php?mode=reviews&type=year&id=".$data->year."\">".$data->year."</a>",
            "FORMAT" => "<a href=\"main.php?mode=reviews&type=format&id=".rawurlencode($data->format)."\">".$data->format."</a>",
            "INFOWIDTH" => $coverelement->image->width
        ));
    }
}

class ReviewElement extends MetadataElement {
    function ReviewElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        $data = $this->metadata->review[0];
        $link_author = str_replace(" ", "+", $data->author);
        $template->assign_vars(array(
            "REVIEW" => $data->review_text,
            "AUTHOR" => "<a href=\"main.php?mode=reviews&type=reviewer&id=".$link_author."\">".$data->author."</a>"
        ));
		if (strtotime($data->date) > 943909200)
            $template->assign_var("REVIEW_DATE", ", ".date("d.m.Y", strtotime($data->date)));
    }
}

class TracksElement extends MetadataElement {
    function TracksElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        $i = 0;
        foreach ($this->metadata->tracks as $disk) {
    		$template->assign_block_vars($this->zone.'DISKINFO', array(
                "DUMMY_VALUE" => "_")
            );
            $diskdata = $this->metadata->disks[$i];
            if (count($this->metadata->tracks) > 1)
				$template->assign_block_vars($this->zone.'DISKINFO.DISKDELIM', array(
					"DISK_ID" => $diskdata->id + 1,
					"FORMAT" => $diskdata->format)
				);
            if ($diskdata->subtitle != "") {
  				$template->assign_block_vars($this->zone.'DISKINFO.DISKSUBTITLE', array(
                    "DISK_ID" => $diskdata->id + 1,
                    "SUBTITLE" => $diskdata->subtitle,
                    "FORMAT" => $diskdata->format.":"
                ));
            }
            $totaltime = 0;
            foreach ($disk as $tracks) {
                if ($tracks->duration == 0)
                    $track_length = "";
                else
    				$track_length = ($tracks->duration-$tracks->duration%60)/60 . ":" . ($tracks->duration%60<10?"0".$tracks->duration%60:$tracks->duration%60);
				$template->assign_block_vars($this->zone.'DISKINFO.TRACKS', array(
					"TRACK_NUMBER" => $tracks->track_number,
					"TRACK_TITLE" => $tracks->title,
					"TRACK_LENGTH" => $track_length)
				);
				$totaltime += $tracks->duration;
            }
            $disk_length = ($totaltime-$totaltime%60)/60 . ":" . ($totaltime%60<10?"0".$totaltime%60:$totaltime%60);
			if($totaltime != 0) {
				$template->assign_block_vars($this->zone.'DISKINFO.TIME', array(
					"TOTAL_TIME" => $disk_length)
				);
			}
            $i++;
        }
    }
}

class SeeAlsoElement extends MetadataElement {
    function SeeAlsoElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        if (count($this->metadata->data) == 0) return;
        $template->assign_block_vars($this->zone."SALSO", array(
            "DUMMY" => "_"
        ));
        foreach ($this->metadata->data as $data) {
            $delim = $data == $this->metadata->data[count($this->metadata->data) - 1] ? "" : ", ";
            $template->assign_block_vars($this->zone."SALSO.SALSOLIST", array(
                "ARTISTLINK" => "<a href=\"main.php?mode=reviews&id=".$data->id."\">".$data->artisttitle."</a>".$delim
            ));
        }
    }
}

class TabsElement extends Element {
    var $tabs;
    var $param;
    function TabsElement($tabs, $param, $zone = "") {
        $this->tabs = $tabs;
        $this->param = $param;
    }
    function prepare(&$template) {
        foreach ($this->tabs->items as $item) {
            $template->assign_block_vars($this->zone."TABS", array( "DUMMY" => "_"));
            if (strtolower($item->name) == $this->tabs->current) {
                $template->assign_block_vars($this->zone."TABS.ACTIVETAB", array(
                    "LINK" => $item->name
                ));
            } else {
                $template->assign_block_vars($this->zone."TABS.NORMALTAB", array(
                    "LINK" => "<a href=\"".$item->address."&param=".$this->param."\">".$item->name."</a>"
                ));
            }
        }
    }
}

class LettersElement extends MetadataElement {
    var $current;
    var $param;
    function LettersElement($metadata, $current, &$param, $zone = "") {
        parent::MetadataElement($metadata, $zone);
        $this->current = $current;
        $this->param = $param;
        if (count($this->metadata->data) == 0) return;
        foreach ($metadata->data as $data) {
            if ($data->letter == $param) return;
        }
        $param = $this->metadata->data[0]->letter;
        $this->param = $param;
    }
    function prepare(&$template) {
        if (count($this->metadata->data) == 0) return;
        $template->assign_block_vars($this->zone."LETTERS", array( "DUMMY" => "_"));
        foreach ($this->metadata->data as $data) {
            $template->assign_block_vars($this->zone."LETTERS.LETTER", array( "DUMMY" => "_"));
            if ($data->letter == $this->param) {
                $template->assign_block_vars($this->zone."LETTERS.LETTER.ACTIVE", array(
                    "LINK" => $data->letter
                ));
            } else {
                $template->assign_block_vars($this->zone."LETTERS.LETTER.NORMAL", array(
                    "LINK" => "<a href=\"main.php?mode=archive&type=".$this->current."&param=".$data->letter."\">".$data->letter."</a>"
                ));
            }
        }
    }
}

class ArchiveElement extends MetadataElement {
    function ArchiveElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        $previous = "";
		$max_release_length = 45;
		$release = "";
        foreach ($this->metadata->data as $data) {
            if ($previous != $data->maintitle) {
                $template->assign_block_vars($this->zone."ARCHIVE", array(
                    "DATA" => "<a href=\"main.php?mode=reviews&type=".$data->type."&id=".urlencode($data->mainouid)."\">".$data->maintitle."</a>",
                ));
                $previous = $data->maintitle;
                $releases = "";
                $delimiter = "";
            }
            if (strlen($releases) < $max_release_length) {
                if (strlen($releases.",".$data->quote.$data->subtitle.$data->quote) < $max_release_length) {
                    $template->assign_block_vars($this->zone."ARCHIVE.RELEASES", array(
                        "RELEASE" => $delimiter.$data->quote."<a href=\"main.php?mode=".$data->submode."&id=".$data->subouid."\">".$data->subtitle."</a>".$data->quote
                    ));
                } else {
                    $template->assign_block_vars($this->zone."ARCHIVE.RELEASES", array(
                        "RELEASE" => "..."
                    ));
                }
                $releases = ",".$data->quote.$releases.$data->subtitle.$data->quote;
                $delimiter = ", ";
            }
        }
    }
}

class GaleryElement extends MetadataElement {
    function GaleryElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function prepare(&$template) {
        $cols = 0;
        $template->assign_block_vars($this->zone."GALLERY", array(
            "DUMMY" => "_"
        ));
        foreach ($this->metadata->data as $data) {
            if ($cols > 2) $cols = 0;
            if ($cols == 0) {
                $template->assign_block_vars($this->zone."GALLERY.ROW", array(
                    "DUMMY" => "_"
                ));
            }
            $image = new Image("photos/".$data->photo);
            $img = "<img src=\"".$image->path."\" width=\"".$image->width."\" height=\"".$image->height."\">";
            $template->assign_block_vars($this->zone."GALLERY.ROW.COL", array(
                "IMG" => $img
            ));
            $cols++;
        }
        for ($i = $cols; $i < 3; $i++) {
            $template->assign_block_vars($this->zone."GALLERY.ROW.COL", array(
                "IMG" => "&nbsp;"
            ));
        }
    }
}

class ProgramsElement extends MetadataElement {
    function ProgramsElement($metadata, $zone = "") {
        parent::MetadataElement($metadata, $zone);
    }
    function get_files($dirname) {
        $files = array();
		if ($dir = @opendir("./programs/".$dirname)) {
			while (($file = readdir($dir)) !== false) {
    			if ($file == "." || $file == "..") continue;
    			$files[] = new FileInfo("./programs/".$dirname."/".$file);
  			}
  			closedir($dir);
		}
		return $files;
    }
    function prepare(&$template) {
        foreach($this->metadata->data as $data) {
			$template->assign_block_vars('program', array(
				"PROGRAM_NAME" => $data->name,
				"DESC" => $data->desc)
			);
            $files = $this->get_files($data->dir);
            foreach ($files as $file) {
				$template->assign_block_vars('program.files', array(
					"DATE" => $file->modificationdate,
					"PATH" => $file->path,
					"NAME" => $file->name,
					"SIZE" => $file->size
                ));
			}
			if ($this->metadata->data[count($this->metadata->data) - 1] != $data) {
				$template->assign_block_vars('program.delimeter', array(
					"DUMMY" => "dummy")
				);
			}
        }
    }
}

class StoryElement extends Element {
    var $filename;
    function StoryElement($filename, $zone = "") {
        parent::Element($zone);
        $this->filename = "./stories/".$filename;
    }
    function prepare(&$template) {
        $fcontents = file($this->filename);
        $i = 0;
        $template->assign_vars(array(
            "STORY" => $fcontents[0]
        ));
        $story = "";
        foreach ($fcontents as $line) {
            if ($i++ < 2) continue;
            if (preg_match("/^[0-9]{4}\n?\r?$/", $line)) {
                $year = $line;
                continue;
            }
            $story = $story.$line;
        }
        $template->assign_vars(array(
            "CONTENTS" => $story,
            "STORY_DATE" => $year
        ));
    }
}

class StoryInfo {
    var $fileinfo;
    var $title = "";
    var $year = "";
    function StoryInfo($filename) {
        $this->fileinfo = new FileInfo($filename);
        $contents = file($filename);
        $this->title = preg_replace("/\n|\r/", "", $contents[0]);
        $this->year = preg_replace("/\n|\r/", "", $contents[count($contents) - 1]);
    }
}

class StoriesElement extends Element {
    var $stories = array();
    function StoriesElement($zone = "") {
        parent::Element($zone);
		if ($dir = @opendir("./stories/")) {
			while (($file = readdir($dir)) !== false) {
    			if ($file == "." || $file == "..") continue;
    			$this->stories[] = new StoryInfo("./stories/".$file);
  			}
  			closedir($dir);
		}
    }
    function prepare(&$template) {
        foreach ($this->stories as $story) {
            $template->assign_block_vars($this->zone."LIST", array(
                "LINK" => "<a href=\"main.php?mode=story&file=".$story->fileinfo->name."\">".$story->title."</a>",
                "YEAR" => $story->year
            ));
        }
    }
}

class FileUploadElement extends Element {
    var $uploaded = false;
    var $file;
    var $path;
    function get_error_description($error) {
        switch ($error) {
            case 1: return "The uploaded file exceeds the upload_max_filesize directive in php.ini.";
            case 2: return "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the html form.";
            case 3: return "The uploaded file was only partially uploaded.";
            case 4: return "No file was uploaded.";
            default: return "There is no error, the file uploaded with success.";
        }
    }
    function FileUploadElement($file, $regex, $password, $path = "", $zone = "") {
        parent::Element($zone);
        $this->file = $file;
        if ($file['name'] == '') return;
        if ($file['error'] > 0) {
            $this->errors[] = 'Error uploading '.$file['name'].": ".$this->get_error_description($file['error']);
            return;
        }
        if (md5($password) != get_upload_password()) {
            $this->errors[] = 'Wrong password for uploading '.$file['name'];
            return;
        }
        if (!preg_match($regex, $file['name'])) {
            $this->errors[] = 'Wrong filename for '.$file['name'];
            return;
        }
		@move_uploaded_file($file['tmp_name'], $_SERVER["DOCUMENT_ROOT"].'/reviews/'.$path.$file['name']);
        $this->uploaded = true;
        $this->path = $path;
    }
    function prepare(&$template) {
        if (!$this->uploaded) return;
        $pathtext = $this->path == "" ? "" : " to ".$this->path;
        $template->assign_block_vars($this->zone."results", array(
            "MESSAGE" => "File ".$this->file['name']." uploaded".$pathtext,
            "FILENAME" => $this->file['name']
        ));
    }
}

class SearchElement extends MetadataElement {
    var $lookin;
    var $searchstr;
    function SearchElement($metadata, $lookin, $searchstr, $zone = "") {
        parent::MetadataElement($metadata);
        $this->lookin = $lookin;
        $this->searchstr = $searchstr;
        if ($searchstr == "") $this->errors[] = "Empty search string";
        if (count($metadata->data) > 100)
            $this->errors[] = "Too many results. Please be more exact";
    }
    function prepare(&$template) {
        if (count($this->errors) > 0) return;
        $template->assign_vars(array(
            $this->zone."SEARCHSTR" => $this->searchstr,
            $this->zone."LOOKIN" => $this->lookin
        ));
        if (count($this->metadata->data) == 0) {
            $template->assign_block_vars($this->zone."RESULTS", array(
                "RESULT" => "Nothing was found"
            ));
        }
        foreach ($this->metadata->data as $data) {
            $template->assign_block_vars($this->zone."RESULTS", array(
                "RESULT" => "<a href=\"main.php?mode=".$data->mode.($data->type==''?"":"&type=".$data->type)."&id=".$data->ouid."\">".$data->maintitle."</a>"
            ));
        }
    }
}

class ActionElement extends Element {
    var $action;
    function ActionElement($action, $zone = "") {
        parent::Element($zone);
        $this->action = $action;
        $this->errors = $action->errors;
    }
    function prepare(&$template) {
        if (count($this->errors) > 0) return;
        $template->assign_vars(array(
            $this->zone."ACTIONNAME" => $this->action->name
        ));
        foreach($this->action->log as $entry) {
            $template->assign_block_vars("LOG", array(
                "entry" => $entry
            ));
        }
    }
}

class ActionsListElement extends MetadataElement {
    function ActionsListElement($metadata) {
        parent::MetadataElement($metadata, "");
    }
    function prepare(&$template) {
        foreach ($this->metadata->actions as $action) {
            $template->assign_block_vars("actionslist", array(
                "ACTION" => $action->name
            ));
        }
    }
}

?>
