<?php

require('config.php');

class Metadata {
    var $dataconnection;
    function Metadata($dataconnection) {
        $this->dataconnection = $dataconnection;
    }
}

class StatisticsMeta extends Metadata {
    var $reviews;
    var $artists;
    var $labels;
    function StatisticsMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT COUNT(review.ouid) AS reviews_count FROM review");
        $this->reviews = $dataconnection->requestdata[0]->reviews_count;
        $dataconnection->query("SELECT COUNT(artists.ouid) AS artists_count FROM artists");
        $this->artists = $dataconnection->requestdata[0]->artists_count;
        $dataconnection->query("SELECT COUNT(labels.ouid) AS labels_count FROM labels");
        $this->labels = $dataconnection->requestdata[0]->labels_count;
    }
}

class RecentReviewsMeta extends Metadata {
    var $data;
    function RecentReviewsMeta($dataconnection, $limit) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT '' AS listby,'' AS listmode,review.ouid AS reviewouid,review.date,release.ouid AS releaseouid,release.title AS releasetitle,artists.ouid AS artistsouid,artists.show_title AS artisttitle,review.release,release.artist,release.type,release.year,release.format ".
            "FROM review,release,artists ".
            "WHERE review.release=release.ouid ".
            "AND artists.ouid=release.artist ".
            "ORDER BY date DESC LIMIT ".$limit);
        $this->data = $dataconnection->requestdata;
    }
}

class ReviewPlanMeta extends Metadata {
    var $reviews;
    function ReviewPlanMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT release FROM review_plan ORDER BY release");
        $this->reviews = $dataconnection->requestdata;
    }
}

class NewsMeta extends Metadata {
    var $news;
    function NewsMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT * FROM news ORDER BY date DESC LIMIT 2");
        $this->news = $dataconnection->requestdata;
    }
}

class DatesMeta extends Metadata {
    var $news;
    function DatesMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT * FROM dates WHERE `date` > NOW() ORDER BY `date`");
        $this->news = $dataconnection->requestdata;
    }
}

class NoticesMeta extends Metadata {
    var $news;
    function NoticesMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT * FROM notices WHERE `date` > NOW() ORDER BY `date`");
        $this->news = $dataconnection->requestdata;
    }
}

class DiscogsMeta extends Metadata {
    var $discogs;
    function DiscogsMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $builder = $dataconnection->get_query_builder();
        $builder->fields[] = "*";
        $builder->tables[] = "discographies";
        $query = $builder->build_select();
        $dataconnection->query($query);
        $this->discogs = $dataconnection->requestdata;
    }
}

class NostalgiaMeta extends Metadata {
    var $photos;
    function NostalgiaMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT photo,photo_desc.description,place FROM photos INNER JOIN photo_desc ON photos.description=photo_desc.ouid");
        $this->photos = $dataconnection->requestdata;
    }
}

class ThanksMeta extends Metadata {
    var $thanks;
    function ThanksMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT * FROM thanks");
        $this->thanks = $dataconnection->requestdata;
    }
}

class FAQMeta extends Metadata {
    var $faq;
    function FAQMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT question,answer FROM faq");
        $this->faq = $dataconnection->requestdata;
    }
}

class ReleaseMeta extends Metadata {
    var $info;
    function ReleaseMeta($dataconnection, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT release.ouid AS releaseouid,release.title AS releasetitle,release.format AS format,release.year AS year,labels.ouid AS labelouid,labels.title AS labeltitle,artists.show_title AS artisttitle,artists.ouid AS artistouid ".
		                     "FROM release,labels,artists ".
		                     "WHERE release.ouid=".$id." ".
		                     "AND release.label=labels.ouid ".
		                     "AND artists.ouid=release.artist");
        $this->info = $dataconnection->requestdata;
    }
}

class ReviewMeta extends Metadata {
    var $review;
    function ReviewMeta($dataconnection, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT * FROM review WHERE ouid=".$id);
        $this->review = $dataconnection->requestdata;
    }
}

class TracksMeta extends Metadata {
    var $tracks = array();
    var $disks = array();
    function TracksMeta($dataconnection, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT * FROM disk WHERE release=".$id." ORDER BY id");
        $disks = $dataconnection->requestdata;
        foreach ($disks as $data) {
            $this->disks[] = $data;
            $dataconnection->query("SELECT tracks_data.title,tracks_data.duration,tracks_data.track_number,disk.id ".
			                      "FROM tracks_data,disk ".
			                      "WHERE tracks_data.diskouid=".$data->ouid." ".
			                      "AND tracks_data.diskouid=disk.ouid ".
			                      "ORDER BY diskouid,tracks_data.track_number");
            $this->tracks[] = $dataconnection->requestdata;
        }
    }
}

class ArtistReviewsMeta extends Metadata {
    var $data;
    function ArtistReviewsMeta($dataconnection, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT 'Artist:' AS listmode,review.ouid AS reviewouid,review.date,release.year AS year,release.type AS type,release.title AS releasetitle,release.format AS format,artists.show_title AS listby,'' AS releaseouid, '' AS artistsouid,'' AS artisttitle ".
			                     "FROM review,release,artists ".
		    	                 "WHERE artists.ouid=".$id." ".
		        	             "AND review.release=release.ouid ".
		            	         "AND artists.ouid=release.artist ".
		                	     "ORDER BY year,release.title");
        $this->data = $dataconnection->requestdata;
    }
}

class LabelReviewsMeta extends Metadata {
    var $data;
    function LabelReviewsMeta($dataconnection, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT 'Label:' AS listmode,review.ouid AS reviewouid,release.year AS year,review.date,release.title AS releasetitle,release.format AS format,artists.show_title AS artisttitle,artists.ouid AS artistsouid,labels.title AS listby ".
			                     "FROM review,release,artists,labels ".
			                     "WHERE review.release=release.ouid ".
			                     "AND labels.ouid=release.label ".
			                     "AND labels.ouid=".$id." ".
			                     "AND artists.ouid=release.artist ".
			                     "ORDER BY year,artists.base_title,release.title");
        $this->data = $dataconnection->requestdata;
    }
}

class ReviewersReviewsMeta extends Metadata {
    var $data;
    function ReviewersReviewsMeta($dataconnection, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT 'Reviewer:' AS listmode,review.ouid AS reviewouid,release.year AS year,review.date,release.title AS releasetitle,release.format AS format,artists.show_title AS artisttitle,artists.ouid AS artistsouid,review.author AS listby ".
			                     "FROM review,release,artists,labels ".
			                     "WHERE review.release=release.ouid ".
			                     "AND labels.ouid=release.label ".
			                     "AND review.author='".urldecode($id)."' ".
			                     "AND artists.ouid=release.artist ".
			                     "ORDER BY year,artists.base_title,release.title");
        $this->data = $dataconnection->requestdata;
    }
}

class FormatsReviewsMeta extends Metadata {
    var $data;
    function FormatsReviewsMeta($dataconnection, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT 'Format:' AS listmode,review.ouid AS reviewouid,release.year AS year,review.date,release.title AS releasetitle,release.format AS format,artists.show_title AS artisttitle,artists.ouid AS artistsouid,release.format AS listby ".
			                     "FROM review,release,artists,labels ".
			                     "WHERE review.release=release.ouid ".
			                     "AND labels.ouid=release.label ".
			                     "AND release.format='".urldecode($id)."' ".
			                     "AND artists.ouid=release.artist ".
			                     "ORDER BY year,artists.base_title,release.title");
        $this->data = $dataconnection->requestdata;
    }
}

class YearsReviewsMeta extends Metadata {
    var $data;
    function YearsReviewsMeta($dataconnection, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT 'Year:' AS listmode,review.ouid AS reviewouid,release.year AS year,review.date,release.title AS releasetitle,release.format AS format,artists.show_title AS artisttitle,artists.ouid AS artistsouid,release.year AS listby ".
			                     "FROM review,release,artists,labels ".
			                     "WHERE review.release=release.ouid ".
			                     "AND labels.ouid=release.label ".
			                     "AND release.year=".urldecode($id)." ".
			                     "AND artists.ouid=release.artist ".
			                     "ORDER BY year,artists.base_title,release.title");
        $this->data = $dataconnection->requestdata;
    }
}

class SeeAlsoMeta extends Metadata {
    var $data;
    function SeeAlsoMeta($dataconnection, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT t1.id,artists.show_title AS artisttitle FROM (SELECT artist2 AS id FROM artists_links WHERE artist1=".$id." UNION SELECT artist1 AS id FROM artists_links WHERE artist2=".$id.") AS t1,artists WHERE t1.id=artists.ouid ORDER BY artisttitle");
        $this->data = $dataconnection->requestdata;
    }
}

class LettersMeta extends Metadata {
    var $data;
    function LettersMeta($dataconnection, $current) {
        parent::Metadata($dataconnection);
        switch ($current) {
            case "artists":
                $dataconnection->query("SELECT DISTINCT letter FROM artists WHERE letter<>'' ORDER BY letter");
                break;
            case "labels":
                $dataconnection->query("SELECT DISTINCT UPPER(LEFT(LTRIM(title),1)) AS letter FROM labels WHERE title<>'' ORDER BY letter");
                break;
            case "reviewers":
                $dataconnection->query("SELECT DISTINCT UPPER(LEFT(author,1)) AS letter FROM review ORDER BY letter");
                break;
        }
        $this->data = $dataconnection->requestdata;
    }
}

class ArchiveMeta extends Metadata {
    var $data;
    function ArchiveMeta($dataconnection, $current, $param) {
        parent::Metadata($dataconnection);
        switch ($current) {
            case "artists":
                $dataconnection->query("SELECT 'artist' AS type,'review' AS submode,'\"' AS quote,review.ouid as subouid,release.title AS subtitle,artists.base_title AS maintitle, artists.ouid AS mainouid ".
	                             "FROM review,release,artists ".
	                             "WHERE review.release=release.ouid ".
	                             "AND release.artist=artists.ouid ".
	                             "AND artists.letter='".$param."' ".
	                             "ORDER BY artists.base_title,subouid");
                break;
            case "labels":
                $dataconnection->query("SELECT DISTINCT 'label' AS type,'reviews' AS submode,'' AS quote,artists.base_title AS subtitle,artists.ouid AS subouid,labels.title AS maintitle,labels.ouid AS mainouid ".
			                     "FROM release,artists,labels ".
			                     "WHERE labels.ouid=release.label ".
		        	             "AND UPPER(LEFT(labels.title,1))='".$param."' ".
		            	         "AND artists.ouid=release.artist ".
		                	     "ORDER BY labels.title");
                break;
            case "reviewers":
                $dataconnection->query("SELECT DISTINCT 'reviewer' AS type,'review' AS submode,'\"' AS quote,review.ouid AS subouid,review.author AS maintitle,review.author AS mainouid,release.title AS subtitle,artists.* ".
	                             "FROM review,release,artists ".
	                             "WHERE review.release=release.ouid ".
	                             "AND release.artist=artists.ouid ".
	                             "AND UPPER(LEFT(author,1))='".$param."' ".
	                             "ORDER BY author");
                break;
            case "formats":
                $dataconnection->query("SELECT DISTINCT 'format' AS type,'review' AS submode,'\"' AS quote,review.ouid AS subouid,release.title AS subtitle,release.format AS maintitle,release.format AS mainouid ".
	                             "FROM review,release,artists ".
	                             "WHERE review.release=release.ouid ".
	                             "AND release.artist=artists.ouid ".
	                             "ORDER BY format");
                break;
            case "years":
                $dataconnection->query("SELECT DISTINCT 'year' AS type,'review' AS submode,'\"' AS quote,review.ouid AS subouid,release.title AS subtitle,release.year AS maintitle,release.year AS mainouid ".
	                             "FROM review,release,artists ".
	                             "WHERE review.release=release.ouid ".
	                             "AND release.artist=artists.ouid ".
	                             "ORDER BY year");
                break;
        }
        $this->data = $dataconnection->requestdata;
    }
}

class GaleryMeta extends Metadata {
    var $data;
    function GaleryMeta($dataconnection, $type, $id) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT photo FROM galeries WHERE type=".$type." AND id=".$id);
        $this->data = $dataconnection->requestdata;
    }
}

class ProgramsMeta extends Metadata {
    var $data;
    function ProgramsMeta($dataconnection) {
        parent::Metadata($dataconnection);
        $dataconnection->query("SELECT `name`,`desc`,`dir` FROM programs ORDER BY name");
        $this->data = $dataconnection->requestdata;
    }
}

class RequestMeta extends Metadata {
    var $data;
    function RequestMeta($dataconnection) {
        parent::Metadata($dataconnection);
    }
    function search_artist($artist) {
        $artist = str_replace("_", " ", $artist);
        $this->dataconnection->query("SELECT ouid FROM artists WHERE show_title LIKE '%".$artist."%' OR base_title LIKE '%".$artist."%'");
        $this->data = $this->dataconnection->requestdata;
        if (count($this->data) > 0) return true; else return false;
    }
    function search_old_artist($artist) {
        $this->dataconnection->query("SELECT ouid FROM artists WHERE old_artist='".$artist."'");
        $this->data = $this->dataconnection->requestdata;
        if (count($this->data) > 0) return true; else return false;
    }
    function search_old_release($artist, $release) {
        $this->dataconnection->query("SELECT review.ouid AS ouid FROM artists,review,release WHERE old_release='".$release."' AND review.release=release.ouid AND release.artist=artists.ouid AND artists.old_artist='".$artist."'");
        $this->data = $this->dataconnection->requestdata;
        if (count($this->data) > 0) return true; else return false;
    }
}

class SearchMeta extends Metadata {
    var $data;
    function SearchMeta($dataconnection, $type, $search) {
        parent::Metadata($dataconnection);
        switch (strtolower($type)) {
            case "artists":
                $dataconnection->query("SELECT DISTINCT 'reviews' AS mode,show_title AS maintitle,ouid FROM artists WHERE show_title LIKE '%".$search."%'");
                break;
            case "labels":
                $dataconnection->query("SELECT DISTINCT 'reviews' AS mode,'label' AS type,title AS maintitle,ouid FROM labels WHERE title LIKE '%".$search."%'");
                break;
            case "releases":
                $dataconnection->query("SELECT DISTINCT 'review' AS mode,CONCAT(artists.show_title, CONCAT(' \"', CONCAT(release.title,'\"'))) AS maintitle,review.ouid AS ouid FROM artists,release,review WHERE release.title LIKE '%".$search."%' AND artists.ouid=release.artist AND review.release=release.ouid");
                break;
            case "reviews":
                $dataconnection->query("SELECT DISTINCT 'review' AS mode,CONCAT(artists.show_title, CONCAT(' \"', CONCAT(release.title,'\"'))) AS maintitle,review.ouid AS ouid FROM artists,release,review WHERE review.review_text LIKE '%".$search."%' AND artists.ouid=release.artist AND review.release=release.ouid");
                break;
            case "tracks":
                $dataconnection->query("SELECT DISTINCT 'review' AS mode,CONCAT(tracks_data.title, CONCAT(', [', CONCAT(artists.show_title, CONCAT(' \"', CONCAT(release.title, '\"]'))))) AS maintitle,review.ouid AS ouid FROM (SELECT diskouid,title FROM tracks_data WHERE tracks_data.title LIKE '%".$search."%' LIMIT 101) AS tracks_data,artists,release,review,disk WHERE artists.ouid=release.artist AND review.release=release.ouid AND release.ouid=disk.release AND tracks_data.diskouid=disk.ouid ORDER by tracks_data.title");
                break;
        }
        $this->data = $dataconnection->requestdata;
    }
}

class UploadNewsMeta extends Metadata {
    var $text;
    var $filename;
    var $password;
    function UploadNewsMeta($dbconnection, $text, $filename, $password) {
        parent::Metadata($dbconnection);
        $this->text = $text;
        $this->filename = $filename;
        $this->password = $password;
    }
    function upload_news() {
        if ($this->text == "") {
            $this->dataconnection->errors[] = "Empty news body";
            return;
        }
        if (md5($this->password) != get_upload_password()) {
            $this->dataconnection->errors[] = "Wrong password for news";
            return;
        }
        $filename = $this->filename;
        if ($filename != "") {
            $filename = "photos/".$filename;
            if (!file_exists($filename)) {
                $this->dataconnection->errors[] = "File ".$filename." does not exist";
                return;
            }
        }
        $builder = $this->dataconnection->get_query_builder();
        $builder->fields = array("body", "date");
        if ($filename != "") $builder->fields[] = "image";
        $builder->tables[] = "news";
		$date = date("Y.m.d H:i:s", time());
        $builder->values = array("body" => $this->text, "date" => $date, "image" => $filename);
        $query = $builder->build_insert();
        $this->dataconnection->execute($query);
    }
}

class ChangesMeta extends Metadata {
    function start_transaction() {
        $this->dataconnection->start();
    }
    function commit_transaction() {
        $this->dataconnection->commit();
    }
    function rollback() {
        $this->dataconnection->rollback();
    }
    function execute_query($query) {
        $this->dataconnection->execute($query);
    }
}

class ActionsMeta extends ChangesMeta {
    var $actions;
    function ActionsMeta($dataconnection) {
        parent::ChangesMeta($dataconnection);
        $builder = $dataconnection->get_query_builder();
        $builder->fields[] = "*";
        $builder->tables[] = "actions";
        $query = $builder->build_select();
        $dataconnection->query($query);
        $this->actions = $dataconnection->requestdata;
    }
}

class NewReviewMeta extends ChangesMeta {
    var $last;
    function insert_entry($fields, $tables, $values, $where) {
        $builder = $this->dataconnection->get_query_builder();
        $builder->fields[] = "ouid";
        $builder->tables = $tables;
        $builder->where = $where;
        $builder->values = $values;
        $query = $builder->build_select();
        $this->dataconnection->query($query);
        $result = $this->dataconnection->requestdata;
        if (count($result) > 0) return $result[0]->ouid;
        $builder->fields = $fields;
        $iquery = $builder->build_insert();
        $this->dataconnection->execute($iquery);
        $this->dataconnection->query($query);
        $result = $this->dataconnection->requestdata;
        return $result[0]->ouid;
    }
    function update_entry($fields, $tables, $values, $where) {
        $builder = $this->dataconnection->get_query_builder();
        $builder->fields[] = "ouid";
        $builder->tables = $tables;
        $builder->where = $where;
        $builder->values = $values;
        $query = $builder->build_select();
        $this->dataconnection->query($query);
        $result = $this->dataconnection->requestdata;
        $builder->fields = $fields;
        if (count($result) > 0) {
            $builder->where = array("ouid");
            $builder->values["ouid"] = $result[0]->ouid;
            $uquery = $builder->build_update();
            $this->dataconnection->execute($uquery);
            return;
        }
        $iquery = $builder->build_insert();
        $this->dataconnection->execute($iquery);
        $this->dataconnection->query($query);
        $result = $this->dataconnection->requestdata;
        return $result[0]->ouid;
    }
    function set_artist($show_title, $base_title, $letter) {
        $fields = array("show_title", "base_title", "letter");
        $tables = array("artists");
        $where = array("show_title");
        $values = array(
            "show_title" => $show_title,
            "base_title" => $base_title,
            "letter" => $letter);
        return $this->insert_entry($fields, $tables, $values, $where);
    }
    function set_label($label) {
        $fields = array("title");
        $tables = array("labels");
        $where = array("title");
        $values = array("title" => $label);
        return $this->insert_entry($fields, $tables, $values, $where);
    }
    function set_release($release, $year, $format, $artist, $type, $label) {
        $fields = array("title", "type", "year", "label", "artist", "format");
        $tables = array("release");
        $where = array("title", "type", "year", "artist");
        settype($artist, "integer");
        settype($year, "integer");
        $values = array(
            "title" => $release,
            "type" => $type,
            "year" => $year,
            "label" => $label,
            "artist" => $artist,
            "format" => $format);
        return $this->insert_entry($fields, $tables, $values, $where);
    }
    function set_disk($release, $disk, $id, $format) {
        $fields = array("subtitle", "release", "format", "id");
        $tables = array("disk");
        $where = array("release", "id");
        settype($release, "integer");
        $values = array(
            "subtitle" => $disk,
            "release" => $release,
            "format" => $format,
            "id" => $id);
        return $this->insert_entry($fields, $tables, $values, $where);
    }
    function set_track($title, $duration, $diskouid, $number) {
        $fields = array("title", "duration", "diskouid", "track_number");
        $tables = array("tracks_data");
        $where = array("title", "duration", "diskouid", "track_number");
        settype($diskouid, "integer");
        settype($number, "integer");
        $values = array(
            "title" => $title,
            "duration" => $duration,
            "diskouid" => $diskouid,
            "track_number" => $number);
        return $this->insert_entry($fields, $tables, $values, $where);
    }
    function set_review($release, $review, $date, $author) {
        $fields = array("release", "review_text", "author", "date");
        $tables = array("review");
        $where = array("release");
        settype($release, "integer");
        $values = array(
            "release" => $release,
            "review_text" => $review,
            "author" => $author,
            "date" => $date);
        $this->update_entry($fields, $tables, $values, $where);
    }
    function set_plan($id, $review) {
        $fields = array("rev_id", "release");
        $tables = array("review_plan");
        $where = array("release");
        settype($id, "integer");
        $values = array("rev_id" => $id, "release" => $review);
        $this->insert_entry($fields, $tables, $values, $where);
    }
    function delete_plan($release) {
        $builder = $this->dataconnection->get_query_builder();
        $builder->field = array("release");
        $builder->tables = array("review_plan");
        $builder->where = array("release");
        $builder->values = array("release" => $release);
        $query = $builder->build_delete();
        $this->execute_query($query);
    }
    function delete_plan_by_id($id) {
        $builder = $this->dataconnection->get_query_builder();
        $builder->field = array("rev_id");
        $builder->tables = array("review_plan");
        $builder->where = array("rev_id");
        settype($id, "integer");
        $builder->values = array("rev_id" => $id);
        $query = $builder->build_delete();
        $this->dataconnection->query($query);
    }
    function set_news($body, $date) {
        $fields = array("body", "date");
        $tables = array("news");
        $where = array("body");
        $values = array("body" => $body, "date" => $date);
        $this->insert_entry($fields, $tables, $values, $where);
    }
    function set_artist_link($artist1, $artist2) {
        $fields = array("artist1", "artist2");
        $tables = array("artists_links");
        $where = array("artist1", "artist2");
        settype($artist1, "integer");
        settype($artist2, "integer");
        $values = array("artist1" => $artist1, "artist2" => $artist2);
        $this->insert_entry($fields, $tables, $values, $where);
    }
    function get_news() {
        $query = "SELECT review.`date`,review.ouid,release.title,artists.show_title FROM review INNER JOIN release ON review.release=release.ouid INNER JOIN artists ON release.artist=artists.ouid ORDER BY `date` DESC LIMIT 5";
        $this->dataconnection->query($query);
        $this->last = $this->dataconnection->requestdata;
    }
}

?>
